﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace EZChart {

  public partial class MainForm : EZ_Builder.UCForms.FormPluginMaster {

    public MainForm() {

      InitializeComponent();

      // Set palette3
      BarChart.Palette = ChartColorPalette.SeaGreen;

      // Set title
      BarChart.Titles.Add("EZ Bar Chart");

      BarChart.Series.Clear();

      // Set Series
      var ChartSeries1 = BarChart.Series.Add("Series1");
      ChartSeries1.Name = "Series1";
      ChartSeries1.ChartType = SeriesChartType.Column;
      ChartSeries1.Points.Add();

      var ChartSeries2 = BarChart.Series.Add("Series2");
      ChartSeries2.ChartType = SeriesChartType.Column;
      ChartSeries2.Points.Add();

      var ChartSeries3 = BarChart.Series.Add("Series3");
      ChartSeries3.ChartType = SeriesChartType.Column;
      ChartSeries3.Points.Add();

      var ChartSeries4 = BarChart.Series.Add("Series4");
      ChartSeries4.ChartType = SeriesChartType.Column;
      ChartSeries4.Points.Add();
    }

    /// This method is called by EZ-Builder when it requests the configuration for your  plugin. 
    /// EZ-Builder will request the configuration when the control is created and when the project is saved.
    /// The data set in this configuration will be serialized and saved in the EZ-Builder user's project.
    /// Custom data may be stored in the STORAGE dictionary.
    /// 
    public override EZ_Builder.Config.Sub.PluginV1 GetConfiguration() {

      return base.GetConfiguration();
    }

    /// This method is called by EZ-Builder when a project is loaded or when the control is added to a project.
    /// Set your default data in here!
    /// The configuration from the user's project file will be set using this method.
    /// *Note: The  plugin must call Base.SetConfiguration(cf) in your override. See the example!
    /// Also note that this is a good place to ensure all required configuration data exists.
    /// In the case of a newer version of your  plugin where different configuration data may be required outside of the users configuration file, set it here.
    /// 
    public override void SetConfiguration(EZ_Builder.Config.Sub.PluginV1 cf) {

      base.SetConfiguration(cf);
    }

    /// This method is called by EZ-Builder when another control sends a command to this control using the EZ-Script ControlCommand() function.
    /// The available ControlCommand() functions for this control should be returned in the GetSupportedControlCommandsMethodsForSlave() override.
    /// 
    public override void SendCommand(string windowCommand, params string[] values) {

      try {

        if (windowCommand.Equals("AddSeries", StringComparison.InvariantCultureIgnoreCase)) {

          if (values.Length == 0)
            throw new Exception("Missing series title to add");

          AddChartSeries(BarChart, values[0]);
        } else if (windowCommand.Equals("RemoveSeries", StringComparison.InvariantCultureIgnoreCase)) {

          if (values.Length == 0)
            throw new Exception("Missing series index to remove");

          int seriesIndex;

          if (!int.TryParse(values[0], out seriesIndex))
            throw new Exception(string.Format("Series index must be a number between 0 and {0}", BarChart.Series.Count - 1));

          if (seriesIndex < 0 || seriesIndex >= BarChart.Series.Count)
            throw new Exception(string.Format("Series index must be a number between 0 and {0}", BarChart.Series.Count - 1));

          RemoveChartSeries(BarChart, seriesIndex);
        } else if (windowCommand.Equals("SetTitle", StringComparison.InvariantCultureIgnoreCase)) {

          if (values.Length == 0)
            throw new Exception("Missing title to set");

          UpdateChartTitle(BarChart, values[0]);
        } else if (windowCommand.Equals("SetSeriesTitle", StringComparison.InvariantCultureIgnoreCase)) {

          if (values.Length < 2)
            throw new Exception("Missing title to set");
          else if (values.Length < 1)
            throw new Exception(string.Format("Missing series index to set (between 0 and {0})", BarChart.Series.Count - 1));

          int seriesIndex;

          if (!int.TryParse(values[0], out seriesIndex))
            throw new Exception(string.Format("Series index must be a number between 0 and {0}", BarChart.Series.Count - 1));

          if (seriesIndex < 0 || seriesIndex >= BarChart.Series.Count)
            throw new Exception(string.Format("Series index must be a number between 0 and {0}", BarChart.Series.Count - 1));

          UpdateChartSeriesTitle(BarChart, seriesIndex, values[1]);
        } else if (windowCommand.Equals("SetSeriesValue", StringComparison.InvariantCultureIgnoreCase)) {

          if (values.Length < 2)
            throw new Exception("Missing series parameter");
          else if (values.Length < 1)
            throw new Exception("Missing series value to set");

          double value;

          if (!double.TryParse(values[1], out value))
            throw new Exception("Value must be a number");

          if (BarChart.Series.FindByName(values[0]) == null)
            throw new Exception(string.Format("The series that you have specified '{0}' is not valid. Please verify the series name is case sensitve", values[0]));

          UpdateChartSeries(BarChart, values[0], value);
        }
      } catch (Exception ex) {

        MessageBox.Show(ex.Message, Text + " Error");
      }
    }

    delegate void RemoveChartSeriesCallBack(Chart control, int seriesIndex);
    public static void RemoveChartSeries(Chart control, int seriesIndex) {

      if (control.InvokeRequired) {

        control.Invoke(new RemoveChartSeriesCallBack(RemoveChartSeries), new object[] { control, seriesIndex });
      } else {

        control.Series.RemoveAt(seriesIndex);
      }
    }

    delegate void AddChartSeriesCallBack(Chart control, string newSeriesTitle);
    public static void AddChartSeries(Chart control, string newSeriesTitle) {

      if (control.InvokeRequired) {

        control.Invoke(new AddChartSeriesCallBack(AddChartSeries), new object[] { control, newSeriesTitle });
      } else {

        var ChartSeries1 = control.Series.Add(newSeriesTitle);
        ChartSeries1.Name = newSeriesTitle;
        ChartSeries1.ChartType = SeriesChartType.Column;
        ChartSeries1.Points.Add();
      }
    }

    delegate void UpdateChartSeriesTitleCallBack(Chart control, int seriesIndex, string newSeriesTitle);
    public static void UpdateChartSeriesTitle(Chart control, int seriesIndex, string newSeriesTitle) {

      if (control.InvokeRequired) {

        control.Invoke(new UpdateChartSeriesTitleCallBack(UpdateChartSeriesTitle), new object[] { control, seriesIndex, newSeriesTitle });
      } else {

        control.Series[seriesIndex].Name = newSeriesTitle;
      }
    }

    delegate void UpdateChartTitleCallBack(Chart control, string title);
    public static void UpdateChartTitle(Chart control, string title) {

      if (control.InvokeRequired) {

        control.Invoke(new UpdateChartTitleCallBack(UpdateChartTitle), new object[] { control, title });
      } else {

        control.Titles[0].Text = title;
      }
    }

    delegate void UpdateChartSeriesCallBack(Chart control, string seriesName, double value);
    public static void UpdateChartSeries(Chart control, string seriesName, double value) {

      if (control.InvokeRequired) {

        control.Invoke(new UpdateChartSeriesCallBack(UpdateChartSeries), new object[] { control, seriesName, value });
      } else {

        control.Series[seriesName].Points.Clear();
        control.Series[seriesName].Points.AddXY(0, value);
      }
    }

    /// <summary>
    /// This method is called by EZ-Builder for the CHEAT SHEET to receive all available ControlCommand() syntax for this control.
    /// When the ControlCommand() is called for this function, see the SendCommand() override.
    /// </summary>
    /// <returns></returns>
    public override object[] GetSupportedControlCommands() {

      List<object> obj = new List<object>();

      obj.Add("SetTitle, [title text]");

      obj.Add("AddSeries, [series title]");

      for (int x = 0; x < BarChart.Series.Count; x++)
        obj.Add(string.Format("RemoveSeries, {0}", x));

      for (int x = 0; x < BarChart.Series.Count; x++)
        obj.Add(string.Format("SetSeriesTitle, {0}, [Series Title]", x));

      foreach (var s in BarChart.Series)
        obj.Add(string.Format("SetSeriesValue, \"{0}\", [number value]", s.Name));

      return obj.ToArray();
    }
  }
}
